@@ -51,18 +51,18 @@
 
 ##' apply
 ##' Computes summary statistics for the spectra of a \code{hyperSpec} object.
-##' 
+##'
 ##' \code{apply} gives the functionality of \code{\link[base]{apply}} for
 ##' \code{hyperSpec} objects.
-##' 
+##'
 ##' The generic functions of group \code{\link[methods]{Math}} are not definded
 ##' for \code{hyperSpec} objects. Instead, \code{apply} can be used. For
 ##' functions like \code{log} that work on scalars, \code{MARGIN = 1 : 2} gives
 ##' the appropriate behaviour.
-##' 
+##'
 ##' \code{spcapply} does the same as \code{apply} with \code{MARGIN = 1}, but
 ##' additionally allows to set a new wavelength axis and adjust the labels.
-##' 
+##'
 ##' \code{wlapply} does the same as \code{apply} with \code{MARGIN = 2}, but
 ##' additionally allows to set a new wavelength axis and adjust the labels.
 ##'
@@ -72,16 +72,16 @@
 ##' @docType methods
 ##' @param X,spc a \code{hyperSpec} object
 ##' @param MARGIN The subscript which the function will be applied over.
-##' 
+##'
 ##' \code{1} indicates rows (\code{FUN} is applied to each spectrum),
-##' 
+##'
 ##' \code{2} indicates columns (\code{FUN} is applied to each wavelength),
-##' 
+##'
 ##' \code{1 : 2} indicates that \code{FUN} should be applied to each single
 ##'   element of the spectra matrix. Note that many basic mathematical
 ##'   functions are already defined for hyperSpec objects (see
 ##'   \code{\link{Math}}).
-##' 
+##'
 ##' If \code{MARGIN} is missing, the whole spectra matrix is handed to
 ##'   \code{FUN}, see also the examples.
 ##' @param FUN function to compute the summary statistics
@@ -98,24 +98,24 @@
 ##' @keywords methods iteration
 ##' @export
 ##' @examples
-##' 
-##' 
+##'
+##'
 ##' plotspc (apply (chondro, 2, range))
-##' 
+##'
 ##' avgflu <- apply (flu, 1, mean,
 ##'                  label.spc = expression (bar (I)),
 ##'                  new.wavelength = mean (wl (flu)))
 ##' avgflu
-##' 
+##'
 ##' flu[[,,405:407]]
 ##' apply (flu, 1:2, "*", -1)[[,,405:407]]
-##' 
+##'
 ##' ## without MARGIN the whole matrix is handed to FUN
 ##' apply (flu [,,405:407], , print) [[]]
-##' 
+##'
 ##' ## whereas MARGIN = 1 : 2 leads to FUN being called for each element separately
 ##' apply (flu [,,405:407], 1 : 2, print) [[]]
-##' 
+##'
 setMethod ("apply", signature = signature (X = "hyperSpec"),
            function (X, MARGIN, FUN, ...,
                      label.wl = NULL, label.spc = NULL, new.wavelength = NULL){
@@ -123,36 +123,36 @@ setMethod ("apply", signature = signature (X = "hyperSpec"),
 
   if (missing (MARGIN)){                # apply for functions that the complete spectra matrix
     ## is easier: tmp <- apply (x, , FUN, ...)
-    ## does: 
+    ## does:
     ## tmp <- x
     ## tmp [[]] <- FUN (x [[]], ...)
 
     X@data$spc <- do.call (FUN, list (X@data$spc, ...))
-    
-  } else if (all (MARGIN == 1 : 2)){    # apply for functions that take scalar arguments. 
+
+  } else if (all (MARGIN == 1 : 2)){    # apply for functions that take scalar arguments.
 
     tmp <- apply (X@data$spc, MARGIN = MARGIN, FUN, ...)
     tmp <- as.numeric (tmp)             # otherwise surprises will be waiting
 
-    dim (tmp) <- dim (X@data$spc)       
+    dim (tmp) <- dim (X@data$spc)
 
     X@data$spc <- tmp
-    
+
   } else {
     ## the usual: for each row / for each column
 
     X@data <- .apply (X@data, MARGIN = MARGIN, FUN = FUN, ...)
 
     if (all (MARGIN == 1)) {
-      
+
       ## if the number of data points per spectrum is changed, the wavelength vector needs to be
       ## adapted, too
-      
+
       if (ncol (X@data$spc) != length (X@wavelength)) {
-        
+
         ## only internal functions here: the validation will fail until the wavelength axis is
         ## adjusted
-        
+
         if (!is.null (new.wavelength)){    # vector with new wavelength is given
           if (is.numeric (new.wavelength)) # either directly,
             .wl (X) <- new.wavelength
@@ -162,9 +162,9 @@ setMethod ("apply", signature = signature (X = "hyperSpec"),
                                                # wavelength vector
           }
         } else if (ncol (X@data$spc) != length (X@wavelength)){
-          wl <- as.numeric (colnames (X@data$spc)) # if not given, try to make from colnames of the 
+          wl <- as.numeric (colnames (X@data$spc)) # if not given, try to make from colnames of the
                                                    # spectra matrix
-        
+
           if (length (wl) != ncol (X@data$spc) || any (is.na (wl)))
             wl <- seq_len (ncol (X@data$spc)) # or just number sequentially
 
@@ -185,27 +185,28 @@ setMethod ("apply", signature = signature (X = "hyperSpec"),
   X
 })
 
+##' @include unittest.R
+.test (.apply) <- function (){
+  context ("apply")
 
+  test_that ("check whether .na.if.different is working correctly", {
+    flu$equal <- 1
+    tmp <- apply (flu, 2, mean)$..
+    expect_equal (is.na (tmp),
+                 structure(c (TRUE, TRUE, FALSE),
+                           .Dim = c(1L, 3L),
+                           .Dimnames = list (NULL, c("filename", "c", "equal")))
+    )
+    expect_equal (tmp$equal, 1)
+  })
 
-.test (apply) <- function (){
-  ## check whether .na.if.different is working correctly
-  rm (flu)
-  flu$equal <- 1
-  tmp <- apply (flu, 2, mean)$..
-  checkEquals (is.na (tmp),
-               structure(c (TRUE, TRUE, FALSE),
-                         .Dim = c(1L, 3L),
-                         .Dimnames = list (NULL, c("file", "c", "equal")))
-               )
-  checkEquals (tmp$equal, 1)
+  test_that ("POSIXct", {
+    flu$ct <- as.POSIXct(Sys.time())
+    expect_equal (apply (flu, 2, mean)$ct, flu$ct [1])
+  })
 
-  ## POSIXct
-  flu$ct <- as.POSIXct(Sys.time()) 
-  checkEquals (apply (flu, 2, mean)$ct, flu$ct [1])
-
-  ## POSIXlt
-  flu$lt <- as.POSIXlt(Sys.time()) 
-  checkEquals (apply (flu, 2, mean)$lt, flu$lt [1])
-
-  rm (flu)
+  test_that ("POSIXlt", {
+    flu$lt <- as.POSIXlt(Sys.time())
+    expect_equal (apply (flu, 2, mean)$lt, flu$lt [1])
+  })
 }
