@@ -9,9 +9,8 @@
 ##' 
 ##' @param spc the \code{hyperSpec} object
 ##' @param newx wavelengh axis to interpolate on
-##' @param enp.target,surface,\dots parameters for \code{\link[stats]{loess}} and
+##' @param enp.target,surface,... parameters for \code{\link[stats]{loess}} and
 ##' \code{\link[stats]{loess.control}}. 
-##' @param short,user,date handed to \code{\link{logentry}}.
 ##' @return a new \code{hyperspec} object.
 ##' @rdname spc-loess
 ##' @export
@@ -29,8 +28,7 @@
 ##' plot (smooth, add = TRUE, col = "blue")
 ##' 
 spc.loess <- function (spc, newx, enp.target = nwl (spc) / 4,
-                       surface = "direct", ...,
-                       short = "spc.loess", user = NULL, date = NULL){
+                       surface = "direct", ...){
 
   .loess <- function (y, x){
     if (all (is.na (y)))
